#!/usr/bin/env python
# filename: pipeline.py


#
# Copyright (c) 2015 Bryan Briney
# License: The MIT license (http://opensource.org/licenses/MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software
# and associated documentation files (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge, publish, distribute,
# sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or
# substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
# BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#


# from __future__ import absolute_import, division, print_function, unicode_literals

import glob
import os
import sys
from typing import Iterable, Optional

from . import log

if sys.version_info[0] > 2:
    STR_TYPES = [
        str,
    ]
# else:
#     STR_TYPES = [str, unicode]


def initialize(log_file, project_dir=None, debug=False):
    """
    Initializes an AbTools pipeline.

    Initialization includes printing the AbTools splash, setting up logging,
    creating the project directory, and logging both the project directory
    and the log location.

    Parameters
    ----------
    log_file : str
        Path to the log file. Required.

    project_dir : str
        Path to the project directory. If not provided,
        the project directory won't be created and the location won't be logged.

    debug : bool
        If ``True``, the logging level will be set to ``logging.DEBUG``.

    Returns
    -------
    logger
        A logger instance.

    """
    print_splash()
    log.setup_logging(log_file, print_log_location=False, debug=debug)
    logger = log.get_logger("pipeline")
    if project_dir is not None:
        make_dir(os.path.normpath(project_dir))
        logger.info("PROJECT DIRECTORY: {}".format(project_dir))
        logger.info("")
    logger.info("LOG LOCATION: {}".format(log_file))
    print("")
    return logger


def make_dir(directory: str) -> None:
    """
    Makes a directory, if it doesn't already exist.

    Parameters
    ----------
    directory : str
        Path to a directory.

    """
    if not os.path.exists(directory):
        os.makedirs(os.path.abspath(directory))


def list_files(directory: str, extension: Optional[str] = None) -> Iterable[str]:
    """
    Lists files in a given directory.

    Parameters
    ----------
    directory : str
        Path to a directory.

    extension : str
        If supplied, only files that contain the specificied extension will be returned.
        Default is ``None``, which returns all files in the directory, regardless of extension.

    Returns
    -------
    Iterable[str]

    """
    if os.path.isdir(directory):
        expanded_dir = os.path.expanduser(directory)
        files = sorted(glob.glob(expanded_dir + "/*"))
    else:
        files = [
            directory,
        ]
    if extension is not None:
        if type(extension) in STR_TYPES:
            extension = [
                extension,
            ]
        files = [
            f
            for f in files
            if any(
                [
                    f.split(".")[-1] in extension,
                    f.split(".")[-1].upper() in extension,
                    f.split(".")[-1].lower() in extension,
                ]
            )
        ]
    return files


def print_splash():
    splash = """
     _    _     _   _ _   _ _       ____  _            _ _            
    / \  | |__ | | | | |_(_) |___  |  _ \(_)_ __   ___| (_)_ __   ___ 
   / _ \ | '_ \| | | | __| | / __| | |_) | | '_ \ / _ \ | | '_ \ / _ \\
  / ___ \| |_) | |_| | |_| | \__ \ |  __/| | |_) |  __/ | | | | |  __/
 /_/   \_\_.__/ \___/ \__|_|_|___/ |_|   |_| .__/ \___|_|_|_| |_|\___|
                                           |_|                        
    """
    print("")
    print(splash)
    print(
        "(c) 2023 Bryan Briney\nDistributed under the MIT License (http://opensource.org/licenses/MIT)"
    )
    print("")



#!/usr/bin/env python
# filename: ridge.py


#
# Copyright (c) 2023 Bryan Briney
# License: The MIT license (http://opensource.org/licenses/MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software
# and associated documentation files (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge, publish, distribute,
# sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or
# substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
# BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#


import warnings

warnings.filterwarnings("ignore")

import itertools
from typing import Iterable, Optional, Union

import numpy as np
import pandas as pd

import seaborn as sns
import matplotlib as mpl
import matplotlib.pyplot as plt

from natsort import natsorted

import pycircos


def ring(
    values: Union[str, dict, Iterable],
    categories: Optional[Union[str, Iterable]] = None,
    data: Optional[pd.DataFrame] = None,
    color: Optional[Union[str, Iterable]] = None,
    palette: Optional[Union[str, Iterable, dict]] = None,
    raxis_range: Optional[tuple] = None,
    linewidth: Optional[float] = 1.0,
    facecolor: Optional[str] = "#FFFFFF",
    label_visible: Optional[bool] = False,
    arc_start: Optional[float] = 0,
    arc_end: Optional[float] = 360,
    title: Optional[str] = None,
    title_fontsize: Optional[float] = 36,
    title_fontweight: Optional[str] = "heavy",
    subtitle: Optional[str] = None,
    subtitle_fontsize: Optional[float] = 20,
    subtitle_fontweight: Optional[str] = "normal",
    show_category_labels: Optional[bool] = True,
    category_label_fontsize: Optional[float] = 16,
    category_label_position_offsets: Optional[dict] = {},
    category_label_match_color: Optional[bool] = True,
    figsize: Optional[Iterable] = [10, 10],
    figfile: Optional[str] = None,
    show: Optional[bool] = False,
) -> Optional[mpl.axes.Axes]:
    """ """
    # process input data


circle = pycircos.Gcircle()

arc = pycircos.Garc(
    arc_id="mAbs",
    size=cc25.shape[0],
    raxis_range=(50, 100),
    linewidth=0,
    facecolor="#FFFFFF",
    label_visible=False,
)

circle.add_garc(arc)
circle.set_garcs(start=0, end=320)

for i, agbc in enumerate(["WA1", "alpha", "beta", "gamma", "kappa"]):
    vals = [1 if v else 0 for v in cc25[f"is_{agbc}"]]
    positions = list(range(len(vals)))
    widths = [0.5] * len(vals)
    raxis = [300 + 75 * i, 350 + 75 * i]
    facecolor = cdict[agbc]
    circle.barplot(
        "mAbs",
        data=vals,
        positions=positions,
        width=widths,
        rlim=[0, 1],
        raxis_range=raxis,
        facecolor=facecolor,
        edgecolor="#ffffff",
        spine=False,
    )


ax = plt.gca()
plt.text(
    0.5,
    0.515,
    "CC25",
    fontsize=36,
    weight="heavy",
    transform=ax.transAxes,
    ha="center",
    va="center",
)
plt.text(
    0.5,
    0.46,
    f"n={cc25.shape[0]}",
    fontsize=20,
    transform=ax.transAxes,
    ha="center",
    va="center",
)

plt.text(
    0.485, 0.66, "WA1", fontsize=16, transform=ax.transAxes, ha="right", va="center"
)
plt.text(
    0.485, 0.7, "alpha", fontsize=16, transform=ax.transAxes, ha="right", va="center"
)
plt.text(
    0.485, 0.74, "beta", fontsize=16, transform=ax.transAxes, ha="right", va="center"
)
plt.text(
    0.485, 0.775, "gamma", fontsize=16, transform=ax.transAxes, ha="right", va="center"
)
plt.text(
    0.485, 0.8125, "kappa", fontsize=16, transform=ax.transAxes, ha="right", va="center"
)

circle.save("./figures/CC25_agbc-ringplot_pairs-only_HSA-9999", format="pdf")
